package TcpExamples;

import java.io.*;
import java.net.*;

/**
 * <p>
 * This program duplicates what happened when telnet (or nc) was invoked in prior examples,
 * directly connecting to an already-running {@link TcpExamples.TcpExample3Server}. 
 * In prior examples, we used telnet (or netcat) to connect to the server application.
 * In this example, we are now writing our own program to make the socket connection.
 * </p>
 * <p>
 * As you will see, when we run this program after we start the server program,
 * we will see a string similar to what telnet printed, sent by the server. 
 * The output at the server shows different socket-pair values, each time the loop iterates.
 * </p>
 *
 * @see TcpExample3Server
 * @see <a href="../../../src/TcpExamples/TcpExample3TerminalLog.txt" target="blank">TcpExample3TerminalLog.txt</a>
 * @see <a href="../../../src/TcpExamples/TcpExample3ServerClientScreenshot.png" target="blank">TcpExample3ServerClientScreenshot.png</a>
 * @see <a href="https://savage.nps.edu/Savage/developers.html#Cygwin" target="blank">Savage Developers Guide: Cygwin</a>
 * @see <a href="https://savage.nps.edu/Savage/developers.html#telnet" target="blank">Savage Developers Guide: telnet</a>
 * @see <a href="https://en.wikipedia.org/wiki/Ipconfig" target="blank">Wikipedia: ipconfig</a>
 * 
 * @author mcgredo
 * @author brutzman@nps.edu
 */
public class TcpExample3Client
{
    /** Default constructor */
    public TcpExample3Client()
    {
        // default constructor
    }

    /** IPv6 String constant for localhost address, similarly IPv4 127.0.0.1
     * @see <a href="https://en.wikipedia.org/wiki/localhost" target="_blank">https://en.wikipedia.org/wiki/localhost</a>
     * @see <a href="https://en.wikipedia.org/wiki/IPv6_address" target="_blank">https://en.wikipedia.org/wiki/IPv6_address</a> 
     */
    public final static String LOCALHOST = "0:0:0:0:0:0:0:1"; //Local host

    /**
     * Program invocation, execution starts here
     * @param args command-line arguments
     */
    @SuppressWarnings("SleepWhileInLoop") // you have been warned!
    public static void main(String[] args)
    {        
        // Local variables/fields
        Socket socket = null;
        InputStream is;
        Reader isr;
        BufferedReader br;
        String serverMessage;
        int clientLoopCount = 0;
        
        try {
            while (true)
            {
                clientLoopCount++; // increment at beginning of loop for reliability
                System.out.println(TcpExample3Client.class.getName() + " creating socket...");

                // We request an IP to connect to ("localhost") and
                // port number at that IP (2317). This establishes
                // a connection to that IP in the form of a Socket
                // object; the server uses a ServerSocket to wait for
                // connections.
                socket = new Socket(LOCALHOST, 2317); // can change hostname (or IP number) to connect to another system

                // Now hook everything up (i.e. set up the streams), Java style:
                is  = socket.getInputStream();
                isr = new InputStreamReader(is);
                br  = new BufferedReader(isr);

                // Read a single line written by the server. We'd
                // do things a bit differently if there were many lines to be read
                // from the server instead of one only.
                serverMessage = br.readLine();
                System.out.println("==================================================");
                       
                System.out.print  ("Client loop " + clientLoopCount + ": ");
                System.out.println("now we're talking!");
                System.out.println("The message the server sent was: '" + serverMessage + "'");
                // socket gets closed, either automatically/silently by this code (or possibly by the server)
                
                Thread.sleep(500l); // slow things down, for example 500l (long) = 500 msec (1/2 second)
                
            } // end while(true) // infinite loops are dangerous, be sure to kill this process!
        } 
        catch (IOException | InterruptedException e )
        {
            System.err.println("Problem with " + TcpExample3Client.class.getName() + " networking:"); // describe what is happening
            System.err.println("Error: " + e);
            
            // Provide more helpful information to user if exception occurs due to running twice at one time
            if (e instanceof java.net.BindException) {
                System.err.println("*** Be sure to stop any other running instances of programs using this port!");
            }
            else if (e instanceof java.net.ConnectException) 
            {
                System.out.println("*** Be sure to start the server before starting the client!");
            }
        }
        finally // occurs after any other activity when shutting down
        {
            try {
                if (socket != null)
                    socket.close();
            } catch (IOException e) {}
            
            // program exit: tell somebody about that happening.  Likely cause: server drops connection.
            System.out.println();
            System.out.println(TcpExample3Client.class.getName() + " exit");
        }
    }
}
